'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const index = require('./index-2f3b9f00.js');
const coordinate = require('./coordinate-5d89d71e.js');
const mapBase = require('./map-base-fd0c9942.js');

const mapEditorCss = ".rl-map-editor{display:inline-block;width:100%;height:100%;overflow:hidden}.rl-map-editor__svg{width:100%;height:100%}.rl-svg{width:100%;height:100%}.rl-svg__marker:focus rect{outline:none;fill:var(--rl-svg__focus-fill, rgba(255, 255, 83, 0.3))}.rl-svg__marker--clickable:hover rect{outline:none;fill:var(--rl-svg__focus-fill, rgba(255, 255, 83, 0.3));cursor:pointer}.rl-svg__marker text{font-size:var(--rl-svg__text-size, 48px);-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}.rl-svg__polygon{fill:var(--rl-svg__polygon-fill, rgba(0, 76, 155, 0.6));stroke:var(--rl-svg__polygon-stroke, #004c9b);stroke-width:var(--rl-svg__polygon-stroke--width, 2)}.rl-svg__polygon:focus{outline:none;fill:var(--rl-svg__focus-fill, rgba(255, 255, 83, 0.3))}.rl-svg__polygon--activated{outline:none;stroke:var(--rl-svg__active-stroke, #c7ab00);stroke-width:var(--rl-svg__active-stroke--width, 4);outline:none;fill:var(--rl-svg__focus-fill, rgba(255, 255, 83, 0.3))}.rl-svg__polygon:hover{outline:none;fill:var(--rl-svg__focus-fill, rgba(255, 255, 83, 0.3));cursor:pointer}.rl-svg__polygon-outline{fill:none;stroke:var(--rl-svg__active-stroke, #c7ab00);stroke-width:var(--rl-svg__active-stroke--width, 2)}.rl-svg__polygon path{outline:none}.rl-svg__rect{fill:var(--rl-svg__rect-fill, rgba(0, 76, 155, 0.6));stroke:var(--rl-svg__rect-stroke, #004c9b);stroke-width:var(--rl-svg__rect-stroke--width, 1)}.rl-svg__rect--activated{outline:none;stroke:var(--rl-svg__active-stroke, #c7ab00);stroke-width:var(--rl-svg__active-stroke--width, 4)}.rl-svg__symbol{fill:var(--rl-svg__symbol-fill, rgba(0, 76, 155, 0.6));stroke:var(--rl-svg__symbol-stroke, #004c9b);stroke-width:var(--rl-svg__symbol-stroke--width, 1);fill-rule:evenodd}.rl-svg__symbol--alt{fill:var(--rl-svg__symbol-fill--alt, rgba(255, 255, 83, 0.5));stroke:var(--rl-svg__symbol-stroke--alt, #c7ab00)}.rl-svg__control{stroke:black;fill:rgba(255, 255, 255, 0.6);stroke-width:2}.rl-svg__control:hover{fill:rgba(192, 192, 192, 0.6);cursor:pointer}.rl-svg__midpoint{stroke:rgba(0, 0, 0, 0.6);fill:rgba(255, 255, 255, 0.3);stroke-width:2}.rl-svg__midpoint:hover{fill:rgba(192, 192, 192, 0.3);cursor:pointer}";

const RLMapEditor = class {
    constructor(hostRef) {
        index.registerInstance(this, hostRef);
        this.elementCreated = index.createEvent(this, "elementCreated", 7);
        this.elementDeselected = index.createEvent(this, "elementDeselected", 7);
        this.elementSelected = index.createEvent(this, "elementSelected", 7);
        this.elementUpdated = index.createEvent(this, "elementUpdated", 7);
        this.elementDeleted = index.createEvent(this, "elementDeleted", 7);
        this.elementDoubleClicked = index.createEvent(this, "elementDoubleClicked", 7);
        /**
         * The original scale factor used to size the SVG so it fits into the SVG and
         * covers the entire SVG.
         */
        this.initialScale = 1;
        /**
         * The array of MapElements currently being displayed.  Created from the
         * `elements` Prop with additional internal information added.
         */
        this.processedElements = [];
        /**
         * The current state of the Map.
         */
        this.state = mapBase.STATES.NORMAL;
        /**
         * Flag indicating if the map image has loaded or not.
         */
        this.imageLoaded = false;
        /**
         * The current size of the SVG element used to display the map.
         */
        this.size = { width: 1, height: 1 };
        /**
         * The factor by which the Map contents are changed to fit within the SVG.
         */
        this.svgScale = 1;
        /**
         * The x,y translation applied to move the map contents within the bounds of
         * the SVG.
         */
        this.svgTransform = new coordinate.Coordinate(0, 0);
        /**
         * An array of the elements that will be displayed on the Map.
         */
        this.elements = [];
        /**
         *  The maximum scale factor.
         */
        this.maxScale = mapBase.DEFAULT_MAX_SCALE;
        /**
         * The minimum scale factor.
         */
        this.minScale = mapBase.DEFAULT_MIN_SCALE;
    }
    /**
     * Handle when the list of specified elements changes. The event details
     * contains the `MapElement` that was deleted.
     */
    onElementsChanged() {
        // if (this.elements !== undefined) {
        this.processedElements = mapBase.parseElements(this.elements, this.svgScale);
        // }
    }
    onMapImageChanged() {
        if (this.mapImage === undefined) {
            return;
        }
        const img = new Image();
        img.src = this.mapImage;
        this.imageLoaded = false;
        img.onload = () => {
            this.imageLoaded = true;
            if (this.imgSize &&
                this.imgSize.width === img.width &&
                this.imgSize.height === img.height) {
                return;
            }
            this.imgSize = { width: img.width, height: img.height };
            this.computeScale();
            this.limits = this.computeLimits();
        };
    }
    componentDidLoad() {
        this.onMapImageChanged();
        this.onElementsChanged();
        this.onResize();
    }
    /**
     * Handles when the user triggers a touch/down event to start interaction with
     * the map.
     * @param e The triggering event.
     */
    onGestureDown(e) {
        this.start = this.last = mapBase.coordinateFromEvent(e, this.root);
        if (this.state === mapBase.STATES.NORMAL) {
            this.state = mapBase.STATES.GESTURE_DOWN;
        }
        if (this.state === mapBase.STATES.ADD_REGION_INIT) {
            if (this.activeElement && this.activeElement instanceof mapBase.MapPolygon) {
                this.activeElement.addPoint(this.toSvgSpace(this.start));
                this.root.forceUpdate();
            }
            this.state = mapBase.STATES.ADD_REGION_FIRST;
        }
        else if (this.state !== mapBase.STATES.ADD_POINT) {
            // Get the ID of the event target, if it is the correct type.
            const id = mapBase.getTargetId(e.target);
            if (id !== undefined) {
                this.targetElement = this.processedElements.find(i => i.id === id);
            }
            if (e.target && e.target instanceof SVGCircleElement) {
                if (e.target.classList.contains('rl-svg__control') ||
                    e.target.classList.contains('rl-svg__midpoint')) {
                    this.targetControl = Number(e.target.dataset.index);
                }
            }
        }
    }
    /**
     * Handles when the user triggers a movement event.
     * @param e The triggering event.
     */
    onGestureMove(e) {
        if (this.state === mapBase.STATES.NORMAL || this.state === mapBase.STATES.ADD_REGION_INIT) {
            return;
        }
        // Convert the coordinate to SVG coordinate space.
        const point = mapBase.coordinateFromEvent(e, this.root);
        const delta = coordinate.Coordinate.difference(point, this.last);
        let dist = 0;
        switch (this.state) {
            case mapBase.STATES.GESTURE_DOWN:
                if (this.start && (this.targetControl !== undefined || this.activeElement && this.targetElement === this.activeElement)) {
                    dist = coordinate.Coordinate.squareDistance(this.start, point);
                    if (dist > mapBase.HYSTERESIS) {
                        this.state = mapBase.STATES.DRAGGING;
                    }
                    break;
                }
            case mapBase.STATES.DRAGGING:
                if (this.targetControl !== undefined && this.activeElement !== undefined && this.activeElement instanceof mapBase.MapPolygon) {
                    if (this.targetControl % 2 === 1) {
                        // If the target control is a midpoint, Add it to the array of
                        // controls.
                        const i = Math.floor(this.targetControl / 2) + 1;
                        this.activeElement.addPoint(coordinate.Coordinate.difference(this.toSvgSpace(point), this.toSvgSpace(this.svgTransform)), i);
                        // Update the targetControl to be the index of the new control.
                        this.targetControl = i * 2;
                    }
                    else {
                        this.activeElement.movePoint(this.toSvgScale(delta), this.targetControl / 2);
                        this.root.forceUpdate();
                    }
                }
                else if (this.targetElement && this.activeElement &&
                    this.activeElement.id === this.targetElement.id) {
                    this.targetElement.move(this.toSvgScale(delta));
                    this.root.forceUpdate();
                }
                else {
                    // Move the canvas otherwise.
                    if (this.limits !== undefined) {
                        this.state = mapBase.STATES.DRAGGING;
                        this.svgTransform = coordinate.Coordinate.sum(this.svgTransform, delta).limit(this.limits).round();
                    }
                }
                break;
            case mapBase.STATES.ADD_REGION_FIRST:
                if (this.start) {
                    dist = coordinate.Coordinate.squareDistance(this.start, point);
                    if (this.activeElement && this.activeElement instanceof mapBase.MapPolygon && dist > mapBase.HYSTERESIS) {
                        this.state = mapBase.STATES.ADD_REGION;
                        this.activeElement.addPoint(this.toSvgSpace(point));
                        this.root.forceUpdate();
                    }
                }
                break;
            case mapBase.STATES.ADD_REGION:
                if (this.activeElement && this.activeElement instanceof mapBase.MapPolygon) {
                    const idx = this.activeElement.points.length - 1;
                    this.activeElement.movePoint(this.toSvgScale(delta), idx);
                    this.root.forceUpdate();
                }
                break;
            default:
        }
        this.last = point;
    }
    /**
     * Handles when the user ends their interaction with the Map.
     * @param e The triggering event.
     */
    onGestureUp(e) {
        // Convert the coordinate to SVG coordinate space.
        const point = mapBase.coordinateFromEvent(e, this.root);
        switch (this.state) {
            case mapBase.STATES.GESTURE_DOWN:
                if (this.targetElement) {
                    if (this.targetElement !== this.activeElement) {
                        this._setActiveElement(this.targetElement);
                    }
                }
                else {
                    this._clearActiveElement();
                    this.elementDeselected.emit(this.activeElement);
                }
                this.state = mapBase.STATES.NORMAL;
                break;
            case mapBase.STATES.DRAGGING:
                const delta = coordinate.Coordinate.difference(point, this.last);
                if (this.activeElement && this.activeElement instanceof mapBase.MapPolygon && this.targetControl !== undefined) {
                    this.activeElement.movePoint(this.toSvgScale(delta), this.targetControl / 2);
                    this.elementUpdated.emit(this.mapElementFromParsedElement(this.activeElement));
                }
                else if (this.targetElement && this.activeElement &&
                    this.activeElement.id === this.targetElement.id) {
                    // Move the targeted element, if there is one.
                    this.activeElement.move(this.toSvgScale(delta));
                    this.elementUpdated.emit(this.mapElementFromParsedElement(this.activeElement));
                }
                else {
                    // Move the canvas otherwise.
                    if (this.limits) {
                        this.svgTransform = coordinate.Coordinate.sum(this.svgTransform, delta).limit(this.limits).round();
                    }
                }
                this.state = mapBase.STATES.NORMAL;
                break;
            case mapBase.STATES.ADD_REGION_FIRST:
                if (this.activeElement && this.activeElement instanceof mapBase.MapPolygon) {
                    this.activeElement.addPoint(this.toSvgSpace(point));
                    this.state = mapBase.STATES.ADD_REGION;
                }
                break;
            case mapBase.STATES.ADD_REGION:
                if (this.activeElement && this.activeElement instanceof mapBase.MapPolygon) {
                    if (this.targetControl === 0) {
                        // Remove the last point, it was used as a placeholder for drawing
                        // a line to the cursor.
                        this.activeElement.removePoint();
                        this.state = mapBase.STATES.NORMAL;
                        this.root.forceUpdate();
                        this.elementCreated.emit(this.mapElementFromParsedElement(this.activeElement));
                    }
                    else {
                        // Add a new point to the active controls array.
                        this.activeElement.addPoint(this.toSvgSpace(point));
                    }
                }
                break;
            case mapBase.STATES.ADD_POINT:
                if (this.activeElement && this.activeElement instanceof mapBase.MapMarker && this.root) {
                    this.activeElement.position = this.toSvgSpace(point);
                    this.state = mapBase.STATES.NORMAL;
                    this.root.forceUpdate();
                    this.elementCreated.emit(this.mapElementFromParsedElement(this.activeElement));
                }
            default:
        }
        this.targetControl = undefined;
        this.targetElement = undefined;
    }
    /**
     * Handles when the user leaves the bounds of the Map entirely.
     * @param e The triggering event.
     */
    onMouseLeave(e) {
        if (this.state === mapBase.STATES.DRAGGING ||
            this.state === mapBase.STATES.GESTURE_DOWN) {
            this.state = mapBase.STATES.NORMAL;
            const point = mapBase.coordinateFromEvent(e, this.root);
            if (this.state === mapBase.STATES.DRAGGING && this.activeElement) {
                const delta = coordinate.Coordinate.difference(point, this.last);
                if (this.activeElement && this.activeElement instanceof mapBase.MapPolygon && this.targetControl !== undefined) {
                    this.activeElement.movePoint(this.toSvgScale(delta), this.targetControl);
                    this.elementUpdated.emit(this.mapElementFromParsedElement(this.activeElement));
                }
                else if (this.targetElement && this.activeElement &&
                    this.activeElement.id === this.targetElement.id) {
                    // Move the targeted element, if there is one.
                    this.activeElement.move(this.toSvgScale(delta));
                    this.elementUpdated.emit(this.mapElementFromParsedElement(this.activeElement));
                }
                else {
                    // Move the canvas otherwise.
                    if (this.limits !== undefined) {
                        this.svgTransform = coordinate.Coordinate.sum(this.svgTransform, delta).limit(this.limits).round();
                    }
                }
                this.state = mapBase.STATES.NORMAL;
            }
        }
    }
    onDoubleClick(e) {
        if (this.state === mapBase.STATES.NORMAL) {
            const id = mapBase.getTargetId(e.target);
            const el = this.elements.find(element => element.id === id);
            if (el !== undefined) {
                this.elementDoubleClicked.emit(el);
            }
        }
    }
    /**
     * Handles when the window is resized.
     */
    onResize() {
        clearTimeout(this.resizeDebounce);
        this.resizeDebounce = setTimeout(_ => {
            // Use a 150ms delay to debounce multiple resize calls and effectively
            // perform the resize action only when the user is 'done' resizing.
            this.computeSvgSize();
            this.computeScale();
            this.limits = this.computeLimits();
            this.svgTransform = this.svgTransform.clone().limit(this.limits);
        }, 150);
    }
    /**
     * Handles when the user performs a mouse wheen event.  The Map will be zoomed
     * in or out depending on the direction of the wheel.
     *
     * @param e The triggering event.
     */
    onWheel(e) {
        const oldPos = this.toSvgSpace(new coordinate.Coordinate(e.clientX, e.clientY));
        if (e.deltaY < 0) {
            if (this.svgScale * 1.05263157895 < this.initialScale * this.maxScale) {
                this.svgScale *= 1.05263157895;
            }
        }
        else {
            if (this.svgScale * 0.95 > this.initialScale * this.minScale) {
                this.svgScale *= 0.95;
            }
        }
        this.limits = this.computeLimits();
        this.processedElements.forEach(el => el.scale = this.svgScale);
        const newPos = this.toSvgSpace(new coordinate.Coordinate(e.clientX, e.clientY));
        const delta = coordinate.Coordinate.difference(newPos, oldPos).scale(this.svgScale);
        this.svgTransform.translate(delta).limit(this.limits).round();
    }
    /**
     * Handle when the user pressses the Enter key.
     * @param e The triggering event
     */
    onEnter(e) {
        if (e.key === 'enter' && e.target && e.target instanceof SVGElement &&
            (e.target.classList.contains('rl-svg__polygon') ||
                e.target.classList.contains('rl-svg__marker'))) {
            const id = Number(e.target.id);
            const el = this.processedElements.find(i => i.id === id);
            this._setActiveElement(el);
        }
    }
    /**
     * Add a new `MapRegion` to the map. Calling this method starts the process.
     * The user must then click numerous times on the map to add points.  Only
     * when clicking again on the original point is the region added.
     */
    async addRegion() {
        this._clearActiveElement();
        this.state = mapBase.STATES.ADD_REGION_INIT;
        this.activeElement = new mapBase.MapPolygon();
        this.activeElement.open = true;
        this.activeElement.scale = this.svgScale;
        this.processedElements = [...this.processedElements, this.activeElement];
    }
    /**
     * Add a new `MapPoint` element to the map.  Calling this method starts the
     * process.  The user must then click somewhere on the map to add the point.
     */
    async addPoint() {
        this._clearActiveElement();
        this.state = mapBase.STATES.ADD_POINT;
        this.activeElement = new mapBase.MapMarker();
        this.processedElements = [...this.processedElements, this.activeElement];
    }
    /**
     * Cancels the current action and returns the map to its default state, ready
     * for futher action.
     */
    async cancelAction() {
        this._clearActiveElement();
        this.state = mapBase.STATES.NORMAL;
    }
    /**
     * Sets the element with the specified ID to active.
     *
     * @param id The ID of the element to set as active.
     */
    async setActiveElement(id) {
        if (this.processedElements.length > 0) {
            this._setActiveElement(this.processedElements.find(i => i.id === id));
        }
    }
    /**
     * Generates a `MapElement` object based on the values in the corresponding
     * `ParsedElement` object.
     *
     * @param el The ParsedElement to convert back to a MapElement
     */
    mapElementFromParsedElement(el) {
        const originalEl = this.elements.find(e => e.id === el.id) || {
            altText: '',
            description: '',
            details: undefined,
            floor: '',
            iconPath: [],
            enabled: true,
            icon: '',
            symbol: '',
        };
        return {
            altText: originalEl.altText,
            description: originalEl.description,
            details: originalEl.details,
            enabled: originalEl.enabled,
            floor: originalEl.floor,
            id: el.id,
            icon: originalEl.icon,
            symbol: originalEl.symbol,
            name: el.name,
            points: el instanceof mapBase.MapPolygon ? mapBase.encodeCoordinates(el.points) : mapBase.encodeCoordinates(el.position),
        };
    }
    /**
     * Clears the currently active element.
     */
    _clearActiveElement() {
        if (this.activeElement) {
            this.activeElement.active = false;
        }
        this.activeElement = undefined;
    }
    /**
     * Updates the limits used to restrict the how far the SVG transform can be
     * moved keeping the map from being panning beyond the edge of the image.
     */
    computeLimits() {
        const { imgSize, size } = this;
        if (imgSize !== undefined) {
            return mapBase.computeLimits(imgSize, size, this.svgScale);
        }
        else {
            return new DOMRect(0, 0, 1, 1);
        }
    }
    /**
     * Computes the scale factor needed to resize the full map image to be as
     * large as possible, and cropping it vertically/horizontally so no empty
     * space exists if the container has different aspect from the image. This is
     * like CSS `background-size: cover` or SVG `aspectRatio xMidYMid slice`.
     */
    computeScale() {
        const { imgSize, size } = this;
        if (imgSize === undefined) {
            this.svgScale = this.initialScale = 1;
        }
        else {
            this.svgScale = this.initialScale =
                Math.max(size.width / imgSize.width, size.height / imgSize.height);
        }
        this.processedElements.forEach(el => el.scale = this.svgScale);
    }
    computeSvgSize() {
        const svg = this.root.querySelector('svg');
        if (svg) {
            const rect = svg.getBoundingClientRect();
            this.size = { width: rect.width, height: rect.height };
        }
    }
    /**
     * Sets the currently active element of the Map.
     *
     * @param el The Element that will be set as the active element.
     */
    _setActiveElement(el) {
        if (!el) {
            this._clearActiveElement();
            this.elementDeselected.emit();
            return;
        }
        this._clearActiveElement();
        this.activeElement = el;
        this.activeElement.active = true;
        const element = this.elements.find(e => this.activeElement && this.activeElement.id === e.id);
        // if (this.elements !== undefined) {
        this.elementSelected.emit(element);
        // }
    }
    /**
     * Scales a given `Coordinate` by the inverse of the current `svgScale`.
     * Returns a new `Coordinate` with its values scaled and rounded.
     * @param c A `Coordinate` scale.
     */
    toSvgScale(c) {
        return c.clone().scale(1 / this.svgScale).round();
    }
    /**
     * Converts a given `Coordinate` object in screen coordinate space to SVG
     * coordinate space.
     * @param c A `Coordinate` to convert to SVG coordinate space.
     */
    toSvgSpace(c) {
        return coordinate.Coordinate.difference(this.toSvgScale(c), this.toSvgScale(this.svgTransform));
    }
    hostData() {
        return {
            class: {
                'rl-map-editor': true,
            },
        };
    }
    __stencil_render() {
        const m = [
            this.svgScale, 0, 0, this.svgScale,
            this.svgTransform.x,
            this.svgTransform.y,
        ];
        const matrix = 'matrix(' + m.join(',') + ')';
        const elements = this.processedElements.map(el => el.render());
        const controls = this.activeElement !== undefined && this.activeElement instanceof mapBase.MapPolygon && this.activeElement.renderControls();
        return (index.h("svg", { class: "rl-svg" }, index.h("g", { transform: matrix }, index.h("image", { xlinkHref: this.mapImage !== undefined ? this.mapImage : undefined }), controls ? [elements, controls] : elements)));
    }
    get root() { return index.getElement(this); }
    static get watchers() { return {
        "elements": ["onElementsChanged"],
        "mapImage": ["onMapImageChanged"]
    }; }
    render() { return index.h(index.Host, this.hostData(), this.__stencil_render()); }
};
RLMapEditor.style = mapEditorCss;

exports.rl_map_editor = RLMapEditor;
