import { Component, Event, Host, Prop, h } from '@stencil/core';
import { SVGContent } from './content';
import { SVGDefs } from './defs';
export class Floorplan {
    constructor() {
        /**
         * The ID of the floorplan to display.
         */
        // @Prop() floorId?: string;
        /**
         * The height for the SVG element.
         */
        this.height = '2400';
        /**
         * The width for the SVG element.
         */
        this.width = '4800';
        /**
         * Setting to use the orthographic variant of the floorplan.
         */
        this.useOrtho = false;
        /**
         * Setting to true adds a viewbox attribute to the SVG element.
         */
        this.useViewbox = false;
        /**
         * The width of the SVG viewbox.
         */
        this.vbHeight = this.height;
        /**
         * The height of the SVG viewbox.
         */
        this.vbWidth = this.width;
    }
    render() {
        const elements = this.svgData !== undefined
            ? this.svgData
            : [{
                    elem: 'text',
                    prefix: '',
                    local: 'text',
                    attrs: {
                        style: 'font-size: 72px; text-anchor: middle;',
                        x: '50%',
                        y: '50%',
                        id: 'placeholder-svg'
                    },
                    content: [{
                            text: 'No SVG Specified',
                        }],
                    type: 'text'
                }];
        return (h(Host, { class: "rl-floorplan" },
            h("svg", { width: this.width, height: this.height, viewBox: this.useViewbox ? `0 0 ${this.vbWidth} ${this.vbHeight}` : undefined, class: "rl-floorplan__svg", onClick: (e) => {
                    if (e.target instanceof SVGElement && e.target.classList.contains('rl-clickable')) {
                        this.rlElementClicked.emit(e.target.id);
                    }
                    else {
                        this.rlElementCleared.emit();
                    }
                } },
                h(SVGDefs, null),
                h("g", null,
                    h(SVGContent, { elements: elements, extra: this.extraElementData })))));
    }
    static get is() { return "rl-floorplan"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() { return {
        "$": ["floorplan.scss"]
    }; }
    static get styleUrls() { return {
        "$": ["floorplan.css"]
    }; }
    static get properties() { return {
        "extraElementData": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "{[key: string]: { [key: string]: string }}",
                "resolved": "undefined | { [key: string]: { [key: string]: string; }; }",
                "references": {}
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "An array of extra attributes that will be assigned to each element based\non the element's code."
            }
        },
        "height": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The height for the SVG element."
            },
            "attribute": "height",
            "reflect": false,
            "defaultValue": "'2400'"
        },
        "width": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The width for the SVG element."
            },
            "attribute": "width",
            "reflect": false,
            "defaultValue": "'4800'"
        },
        "svgData": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "SVGEl[]",
                "resolved": "SVGEl[] | undefined",
                "references": {
                    "SVGEl": {
                        "location": "import",
                        "path": "../../interface"
                    }
                }
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "An SVG encoded into a JSON object that will be displayed."
            }
        },
        "useOrtho": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Setting to use the orthographic variant of the floorplan."
            },
            "attribute": "use-ortho",
            "reflect": false,
            "defaultValue": "false"
        },
        "useViewbox": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Setting to true adds a viewbox attribute to the SVG element."
            },
            "attribute": "use-viewbox",
            "reflect": false,
            "defaultValue": "false"
        },
        "vbHeight": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The width of the SVG viewbox."
            },
            "attribute": "vb-height",
            "reflect": false,
            "defaultValue": "this.height"
        },
        "vbWidth": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The height of the SVG viewbox."
            },
            "attribute": "vb-width",
            "reflect": false,
            "defaultValue": "this.width"
        }
    }; }
    static get events() { return [{
            "method": "rlElementClicked",
            "name": "rlElementClicked",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "Event fired when an element in the SVG is clicked (if it is clickable).\nDetails will be the `id` of the clicked element."
            },
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            }
        }, {
            "method": "rlElementCleared",
            "name": "rlElementCleared",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "Event fired when the SVG is clicked but no specific element is targeted,\nthat is, the active element is cleared."
            },
            "complexType": {
                "original": "void",
                "resolved": "void",
                "references": {}
            }
        }]; }
}
