import { r as registerInstance, c as createEvent, h, g as getElement, H as Host } from './index-f121c89a.js';
import { C as Coordinate } from './coordinate-997d4413.js';
import { S as STATES, D as DEFAULT_MAX_SCALE, a as DEFAULT_MIN_SCALE, p as parseElements, c as coordinateFromEvent, H as HYSTERESIS, b as computeLimits, g as getTargetId } from './map-base-4f0e515f.js';
var mapCss = "@-webkit-keyframes fade-in{0%{opacity:0}100%{opacity:1}}@keyframes fade-in{0%{opacity:0}100%{opacity:1}}.rl-map{display:inline-block;width:100%;height:100%;opacity:0;overflow:hidden}.rl-map--loaded{opacity:1;-webkit-animation:fade-in 180ms ease-in;animation:fade-in 180ms ease-in}.rl-svg{width:100%;height:100%}.rl-svg__marker:focus rect{outline:none;fill:var(--rl-svg__focus-fill, rgba(255, 255, 83, 0.3))}.rl-svg__marker--clickable:hover rect{outline:none;fill:var(--rl-svg__focus-fill, rgba(255, 255, 83, 0.3));cursor:pointer}.rl-svg__marker text{font-size:var(--rl-svg__text-size, 48px);-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}.rl-svg__polygon{fill:var(--rl-svg__polygon-fill, rgba(0, 76, 155, 0.6));stroke:var(--rl-svg__polygon-stroke, #004c9b);stroke-width:var(--rl-svg__polygon-stroke--width, 2)}.rl-svg__polygon:focus{outline:none;fill:var(--rl-svg__focus-fill, rgba(255, 255, 83, 0.3))}.rl-svg__polygon--activated{outline:none;stroke:var(--rl-svg__active-stroke, #c7ab00);stroke-width:var(--rl-svg__active-stroke--width, 4);outline:none;fill:var(--rl-svg__focus-fill, rgba(255, 255, 83, 0.3))}.rl-svg__polygon:hover{outline:none;fill:var(--rl-svg__focus-fill, rgba(255, 255, 83, 0.3));cursor:pointer}.rl-svg__polygon-outline{fill:none;stroke:var(--rl-svg__active-stroke, #c7ab00);stroke-width:var(--rl-svg__active-stroke--width, 4)}.rl-svg__polygon path{outline:none}.rl-svg__rect{fill:none;stroke:none}.rl-svg__rect--activated{outline:none;stroke:var(--rl-svg__active-stroke, #c7ab00);stroke-width:var(--rl-svg__active-stroke--width, 4)}.rl-svg__symbol{fill:var(--rl-svg__symbol-fill, rgba(0, 76, 155, 0.6));stroke:var(--rl-svg__symbol-stroke, #004c9b);stroke-width:var(--rl-svg__symbol-stroke--width, 1);fill-rule:evenodd}.rl-svg__symbol--alt{fill:var(--rl-svg__symbol-fill--alt, rgba(255, 255, 83, 0.5));stroke:var(--rl-svg__symbol-stroke--alt, #c7ab00)}";
var RLMap = /** @class */ (function () {
    function RLMap(hostRef) {
        registerInstance(this, hostRef);
        this.elementDeselected = createEvent(this, "elementDeselected", 7);
        this.elementSelected = createEvent(this, "elementSelected", 7);
        /**
         * The array of MapElements currently being displayed.  Created from the
         * `elements` Prop with additional internal information added.
         */
        this.processedElements = [];
        /**
         * The original scale factor used to size the SVG so it fits into the SVG and
         * covers the entire SVG.
         */
        this.initialScale = 1;
        /**
         * The current state of the Map.
         */
        this.state = STATES.NORMAL;
        /**
         * Flag indicating if the map image has loaded or not.
         */
        this.imageLoaded = false;
        /**
         * The current size of the SVG element used to display the map.
         */
        this.size = { width: 1, height: 1 };
        /**
         * The factor by which the Map contents are changed to fit within the SVG.
         */
        this.svgScale = 1;
        /**
         * The x,y translation applied to move the map contents within the bounds of
         * the SVG.
         */
        this.svgTransform = new Coordinate(0, 0);
        /**
         *  The maximum scale factor.
         */
        this.maxScale = DEFAULT_MAX_SCALE;
        /**
         * The minimum scale factor.
         */
        this.minScale = DEFAULT_MIN_SCALE;
    }
    RLMap.prototype.onActiveElementIdChanged = function () {
        var _this = this;
        if (this.processedElements.length > 0) {
            this._setActiveElement(this.processedElements.find(function (i) { return i.id === _this.activeElementId; }), false);
        }
    };
    RLMap.prototype.onMapImageChanged = function () {
        var _this = this;
        if (this.mapImage === undefined) {
            return;
        }
        var img = new Image();
        img.src = this.mapImage;
        this.imageLoaded = false;
        img.onload = function () {
            _this.imageLoaded = true;
            if (_this.imgSize &&
                _this.imgSize.width === img.width &&
                _this.imgSize.height === img.height) {
                return;
            }
            _this.imgSize = { width: img.width, height: img.height };
            _this.computeScale();
            _this.limits = _this.computeLimits();
        };
    };
    /**
     * Handle when the list of specified elements changes.
     */
    RLMap.prototype.onElementsChanged = function () {
        // if (this.elements !== undefined) {
        this.processedElements = parseElements(this.elements);
        // }
    };
    RLMap.prototype.componentDidLoad = function () {
        this.onMapImageChanged();
        this.onElementsChanged();
        this.onResize();
    };
    /**
     * Handles when the user triggers a touch/down event to start interaction with
     * the map.
     * @param e The triggering event.
     */
    RLMap.prototype.onGestureDown = function (e) {
        this.start = this.last = coordinateFromEvent(e, this.root);
        if (this.state === STATES.NORMAL) {
            this.state = STATES.GESTURE_DOWN;
        }
        // Get the ID of the event target, if it is the correct type.
        var id = getTargetId(e.target);
        if (id !== undefined) {
            this.targetElement = this.processedElements.find(function (i) { return i.id === id; });
        }
    };
    /**
     * Handles when the user triggers a movement event.
     * @param e The triggering event.
     */
    RLMap.prototype.onGestureMove = function (e) {
        if (this.state === STATES.NORMAL) {
            return;
        }
        // Convert the coordinate to SVG coordinate space.
        var point = coordinateFromEvent(e, this.root);
        switch (this.state) {
            case STATES.GESTURE_DOWN:
                var dist = Coordinate.squareDistance(this.start, point);
                if (dist > HYSTERESIS) {
                    this.state = STATES.DRAGGING;
                }
                break;
            case STATES.DRAGGING:
                if (this.limits !== undefined) {
                    var delta = Coordinate.difference(point, this.last);
                    this.svgTransform = Coordinate.sum(this.svgTransform, delta).limit(this.limits).round();
                }
                break;
        }
        this.last = point;
    };
    /**
     * Handles when the user ends their interaction with the Map.
     * @param e The triggering event.
     */
    RLMap.prototype.onGestureUp = function (e) {
        var point = coordinateFromEvent(e, this.root);
        switch (this.state) {
            case STATES.DRAGGING:
                if (this.limits !== undefined) {
                    var delta = Coordinate.difference(point, this.last);
                    this.svgTransform = Coordinate.sum(this.svgTransform, delta).limit(this.limits).round();
                }
                break;
            case STATES.GESTURE_DOWN:
                if (this.targetElement) {
                    if (this.targetElement !== this.activeElement && this.targetElement.clickable) {
                        this._setActiveElement(this.targetElement);
                        this.targetElement = undefined;
                    }
                }
                else {
                    if (this.activeElement) {
                        this.elementDeselected.emit(this.activeElement);
                        this._clearActiveElement();
                    }
                }
                break;
        }
        this.state = STATES.NORMAL;
    };
    /**
     * Handles when the user leaves the bounds of the Map entirely.
     * @param e The triggering event.
     */
    RLMap.prototype.onMouseLeave = function (e) {
        if (this.state === STATES.DRAGGING ||
            this.state === STATES.GESTURE_DOWN) {
            this.state = STATES.NORMAL;
            var point = coordinateFromEvent(e, this.root);
            if (this.state === STATES.DRAGGING && this.limits !== undefined) {
                var delta = Coordinate.difference(point, this.last);
                this.svgTransform = Coordinate.sum(this.svgTransform, delta).limit(this.limits).round();
                this.state = STATES.NORMAL;
            }
        }
    };
    /**
     * Handles when the user performs a resize action.
     */
    RLMap.prototype.onResize = function () {
        var _this = this;
        clearTimeout(this.resizeDebounce);
        this.resizeDebounce = setTimeout(function (_) {
            // Use a 150ms delay to debounce multiple resize calls and effectively
            // perform the resize action only when the user is 'done' resizing.
            _this.computeSvgSize();
            _this.computeScale();
            _this.limits = _this.computeLimits();
            _this.svgTransform = _this.svgTransform.clone().limit(_this.limits);
        }, 150);
    };
    /**
     * Handles when the user performs a mouse wheen event.  The Map will be zoomed
     * in or out depending on the direction of the wheel.
     *
     * @param e The triggering event.
     */
    RLMap.prototype.onWheel = function (e) {
        var oldPos = this.toSvgSpace(new Coordinate(e.clientX, e.clientY));
        if (e.deltaY < 0) {
            if (this.svgScale * 1.05263157895 < this.initialScale * this.maxScale) {
                this.svgScale *= 1.05263157895;
            }
        }
        else {
            if (this.svgScale * 0.95 > this.initialScale * this.minScale) {
                this.svgScale *= 0.95;
            }
        }
        this.limits = this.computeLimits();
        var newPos = this.toSvgSpace(new Coordinate(e.clientX, e.clientY));
        var delta = Coordinate.difference(newPos, oldPos).scale(this.svgScale);
        this.svgTransform.translate(delta).limit(this.limits).round();
    };
    RLMap.prototype.onEnter = function (e) {
        if (e.key === 'enter' && e.target && e.target instanceof SVGElement &&
            (e.target.classList.contains('rl-map-polygon') ||
                e.target.classList.contains('rl-map-marker'))) {
            var id_1 = Number(e.target.id);
            var el = this.processedElements.find(function (i) { return i.id === id_1; });
            this._setActiveElement(el);
        }
    };
    /**
     * Clears the currently active element.
     */
    RLMap.prototype._clearActiveElement = function () {
        if (this.activeElement) {
            this.activeElement.active = false;
        }
        this.activeElement = undefined;
    };
    /**
     * Updates the limits used to restrict the how far the SVG transform can be
     * moved keeping the map from being panning beyond the edge of the image.
     */
    RLMap.prototype.computeLimits = function () {
        var _a = this, imgSize = _a.imgSize, size = _a.size;
        if (imgSize !== undefined) {
            return computeLimits(imgSize, size, this.svgScale);
        }
        else {
            return new DOMRect(0, 0, 1, 1);
        }
    };
    /**
     * Computes the scale factor needed to resize the full map image to be as
     * large as possible, and cropping it vertically/horizontally so no empty
     * space exists if the container has different aspect from the image. This is
     * like CSS `background-size: cover` or SVG `aspectRatio xMidYMid slice`.
     */
    RLMap.prototype.computeScale = function () {
        var _a = this, imgSize = _a.imgSize, size = _a.size;
        if (imgSize === undefined) {
            this.svgScale = this.initialScale = 1;
        }
        else {
            this.svgScale = this.initialScale =
                Math.max(size.width / imgSize.width, size.height / imgSize.height);
        }
    };
    /**
     * Determines the current size of the SVG element serving as the container
     * for the entire map.
     */
    RLMap.prototype.computeSvgSize = function () {
        var root = this.root.querySelector('svg');
        var rect = root && root.getBoundingClientRect();
        if (rect !== null) {
            this.size = { width: rect.width, height: rect.height };
        }
    };
    /**
     * Sets the currently active element of the Map.
     *
     * @param el The Element that will be set as the active element.
     */
    RLMap.prototype._setActiveElement = function (el, shouldEmit) {
        if (shouldEmit === void 0) { shouldEmit = true; }
        if (!el || (this.activeElement && el.id === this.activeElement.id)) {
            return;
        }
        this._clearActiveElement();
        this.activeElement = el;
        this.activeElement.active = true;
        // Emit an event with the original (un-parsed) element as the new 'active'
        // element.  Don't emit an event if the flag is set.
        if (shouldEmit) {
            this.elementSelected.emit(this.elements[this.activeElement.id]);
        }
    };
    /**
     * Scales a given `Coordinate` by the inverse of the current `svgScale`.
     * Returns a new `Coordinate` with its values scaled and rounded.
     * @param c A `Coordinate` scale.
     */
    RLMap.prototype.toSvgScale = function (c) {
        return c.clone().scale(1 / this.svgScale).round();
    };
    /**
     * Converts a given `Coordinate` object in screen coordinate space to SVG
     * coordinate space.
     * @param c A `Coordinate` to convert to SVG coordinate space.
     */
    RLMap.prototype.toSvgSpace = function (c) {
        return Coordinate.difference(this.toSvgScale(c), this.toSvgScale(this.svgTransform));
    };
    RLMap.prototype.hostData = function () {
        return {
            class: {
                'rl-map': true,
                'rl-map--loaded': this.imageLoaded,
            },
        };
    };
    RLMap.prototype.__stencil_render = function () {
        var m = [
            this.svgScale, 0, 0, this.svgScale,
            this.svgTransform.x,
            this.svgTransform.y,
        ];
        var matrix = 'matrix(' + m.join(',') + ')';
        return (h("svg", { class: "rl-svg" }, h("g", { transform: matrix }, h("image", { xlinkHref: this.mapImage !== undefined ? this.mapImage : undefined }), this.processedElements.map(function (el) { return el.render(); }))));
    };
    Object.defineProperty(RLMap.prototype, "root", {
        get: function () { return getElement(this); },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(RLMap, "watchers", {
        get: function () {
            return {
                "activeElementId": ["onActiveElementIdChanged"],
                "mapImage": ["onMapImageChanged"],
                "elements": ["onElementsChanged"]
            };
        },
        enumerable: false,
        configurable: true
    });
    RLMap.prototype.render = function () { return h(Host, this.hostData(), this.__stencil_render()); };
    return RLMap;
}());
RLMap.style = mapCss;
export { RLMap as rl_map };
