import { iMapElementOptions } from '../interface';
/**
 * The base class for the different kinds of elements that can be rendered on a
 * Map.
 */
export declare class MapElement {
    _active: boolean;
    /**
     * Indicates whether this `MapElement` is currently selected by the user. When
     * active, the `MapElement` is styled differently.  Defaults to `false`.
     */
    set active(isActive: boolean);
    get active(): boolean;
    _clickable: boolean;
    /**
     * Indicates whether this `MapElement` receives mouse and touch events.
     * Defaults to `true`.
     */
    set clickable(isActive: boolean);
    get clickable(): boolean;
    _id: number;
    /**
     * The unique identifier for this `MapElement`.
     */
    set id(newCode: number);
    get id(): number;
    _name: string;
    /**
     * The name of this `MapElement`.
     */
    set name(newName: string);
    get name(): string;
    _scale: number;
    /**
     * The scale of this `MapElement`.
     */
    set scale(newScale: number);
    get scale(): number;
    _visible: boolean;
    /**
     * Indicates if this `MapElement` is currently visible. Defaults to `true`.
     */
    set visible(isVisible: boolean);
    get visible(): boolean;
    _zIndex: number;
    /**
     * The z-index used to draw this `MapElement`.  Defaults to 1.
     */
    set zIndex(isVisible: number);
    get zIndex(): number;
    /**
     * Create a new `MapElement`.
     *
     * @param id A value to set as the ID of the `MapElement`.
     */
    constructor(opts?: iMapElementOptions);
}
