import { EventEmitter } from '../../stencil-public-runtime';
import { SVGEl } from '../../interface';
export declare class Floorplan {
    /**
     * Event fired when an element in the SVG is clicked (if it is clickable).
     * Details will be the `id` of the clicked element.
     */
    rlElementClicked: EventEmitter<string>;
    /**
     * Event fired when the SVG is clicked but no specific element is targeted,
     * that is, the active element is cleared.
     */
    rlElementCleared: EventEmitter<void>;
    /**
     * An array of extra attributes that will be assigned to each element based
     * on the element's code.
     */
    extraElementData?: {
        [key: string]: {
            [key: string]: string;
        };
    };
    /**
     * The ID of the floorplan to display.
     */
    /**
     * The height for the SVG element.
     */
    height: string;
    /**
     * The width for the SVG element.
     */
    width: string;
    /**
     * An SVG encoded into a JSON object that will be displayed.
     */
    svgData?: SVGEl[];
    /**
     * Setting to use the orthographic variant of the floorplan.
     */
    useOrtho: boolean;
    /**
     * Setting to true adds a viewbox attribute to the SVG element.
     */
    useViewbox: boolean;
    /**
     * The width of the SVG viewbox.
     */
    vbHeight: string;
    /**
     * The height of the SVG viewbox.
     */
    vbWidth: string;
    render(): any;
}
