import { Coordinate } from '../../utils/coordinate';
/**
 * A component that provides pan and zoom functionality for the content within
 * it.
 */
export declare class PanZoom {
    /**
     * The number of active pointer events.
     */
    private _active;
    /**
     * The initial scale of the content.  Any further scaling is done relative
     * to this value.  This is needed so content can be scaled to fit the size
     * of the `PanZoom` component.
     */
    private _initScale;
    /**
     * The initial size of the content to be moved.
     */
    private _initSize?;
    /**
     * Bounds used to define how far the content can be moved.
     */
    private _limits;
    /**
     * The element used to perform the transform.
     */
    private _tEl?;
    root: HTMLRlPanZoomElement;
    /**
     * The offset from the origin to move the object.
     */
    _d: Coordinate;
    /**
     * All the old pointer event locations.
     */
    _lastPoints: Coordinate[];
    /**
     * The scale factor applied to the object.
     */
    _scale: number;
    /**
     * If true, the content of the `PanZoom` will be scaled so that it fills
     * the parent container initially.
     */
    scaled: boolean;
    /**
     * If true, the content of the `PanZoom` will be able to move beyond the
     * bounds of the parent container.
     */
    unbound: boolean;
    /**
     * The smallest scale factor allowed when scaling the content.
     */
    minScale: number;
    /**
     * The largest factor allowed when scaling the content.  Calculated as a
     * factor of the original size.  So a value of `3` would limit the scale to
     * three times the original size.
     */
    maxScale: number;
    /**
     * Lifecycle event called when the component has been loaded and added to
     * the DOM tree.
     */
    componentDidLoad(): void;
    /**
     * Handle when the mouse or a touch goes outside the bounds of the `PanZoom`
     * component.
     */
    onPointerLeave: (event: MouseEvent | TouchEvent) => void;
    /**
     * Handle when a mouse button is pressed or a touch event starts.
     */
    onPointerDown: (event: MouseEvent | TouchEvent) => void;
    /**
     * Handle when a touch or mouse moves.
     */
    onPointerMove: (event: MouseEvent | TouchEvent) => void;
    /**
     * Handle when a touch ends or a mouse button is released to end the pan.
     */
    onPointerUp: (event: MouseEvent | TouchEvent) => void;
    /**
     * Handle a `WheelEvent` to scale the contents.
     */
    onWheel: (event: WheelEvent) => void;
    /**
     * Force a recalculation of the size of the content.
     */
    resize(): Promise<void>;
    /**
     * Computes the limits used to constrain how far the content can move.
     */
    private _computeLimits;
    /**
     * Compute the initial scale used to resize the content to best fit within
     * the bounds of the `PanZoom` component.
     */
    private _setInitialScale;
    /**
     * Lifecycle method called when the component needs to redraw.
     */
    render(): any;
}
